// TLfit
// by Alex Matulich, Unicorn Research Corporation, 29 August 2008
//
// This indicator shows how well regression lines of all lengths
// (between a specified range of lengths) fit the price data. The
// quality of fit is plotted as a color or shade corresponding to
// the trendline length on the vertical axis.
//
// The indicator isn't terribly useful except perhaps to show the
// simultaneous ending of multiple trends, which may indicate a
// new direction for the market.
//
// The purpose of this indicator is really to demonstrate two things:
// 1. Creation of numerous instances of aliases for a method,
//    and calling them in a loop.
// 2. Painting stuff on the indicator panel by overriding the Plot() method.

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
    /// </summary>
    [Description("Displays color map of the quality of fit of all trendlines within a specified range of lengths.")]
    public class TLfit : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int minLength = 5; // Default setting for MinLength
            private int maxLength = 40; // Default setting for MaxLength
			private bool grayscale = true; // Default setting for Grayscale
        // User defined variables (add any user defined variables below)
			private LinRegSlopeSFX[] TL;
			private double[] errs;
			private DataSeries[] TLerrcolor;
			private int nn;

		// Color setup
			private int ncolors = 32; // number of colors to create in palette
			private SolidBrush[] brushcolor; // array to hold color definitions
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			BarsRequired = minLength;
			
			nn = maxLength - minLength + 1;
			TL = new LinRegSlopeSFX[nn]; // nn instances of this method
			TLerrcolor = new DataSeries[nn];
			errs = new double[nn];
			for (int i = 0; i < nn; ++i) {
				TL[i] = LinRegSlopeSFX(Input, true, minLength+i, 3); // create method instance
				TLerrcolor[i] = new DataSeries(this); // create series to hold result
			}
			
			// COLOR SETUP
			// interpolate ncolors (default 32) colors from the R,G,B arrays
			int[] R, G, B;
			if (grayscale) {
				R = new int[] { 0xff, 0x00 }; // we need only start and end for grayscale
				G = new int[] { 0xff, 0x00 };
				B = new int[] { 0xff, 0x00 };
			} else {
				//              white yellow red magnta blue blugrn green dkgrn black
				R = new int[] { 0xff, 0xdd, 0xee, 0x99, 0x00, 0x00, 0x00, 0x00, 0x00 };
				G = new int[] { 0xff, 0xee, 0x00, 0x00, 0x22, 0x66, 0x77, 0x33, 0x00 };
				B = new int[] { 0xff, 0x00, 0x00, 0xaa, 0xee, 0xbb, 0x11, 0x00, 0x00 };
			}
			int rgbidx, red, green, blue, arraycolors = R.Length;
			double interp, frac;
			brushcolor = new SolidBrush[ncolors];
			for (int i = 0; i < ncolors; ++i) {
				rgbidx = (int)(interp = (arraycolors-1)*i/(double)ncolors);
				frac = interp - Math.Floor(interp);
				red = (int)((R[rgbidx+1] - R[rgbidx]) * frac + R[rgbidx] + 0.5);
				green = (int)((G[rgbidx+1] - G[rgbidx]) * frac + G[rgbidx] + 0.5);
				blue = (int)((B[rgbidx+1] - B[rgbidx]) * frac + B[rgbidx] + 0.5);
				brushcolor[i] = new SolidBrush(Color.FromArgb(0x00ff, red, green, blue)); 
			}
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < minLength) return;
			int i;
			double maxerr = -9.99e+30, minerr = 9.99e+30;
			for (i = 0; i < nn; ++i) {
				errs[i] = TL[i][0]; // invoking TL[i] here automatically calls the i-th instance of LinRegSlopeSFX
				if (errs[i] > maxerr) maxerr = errs[i];
				if ((errs[i] < minerr && errs[i] > 0.0) || i==0) minerr = errs[i];
				//erridx[i] = i;
			}
			for (i = 0; i < nn; ++i) {
				if (errs[i] < minerr) errs[i] = maxerr; // fix zeros
				TLerrcolor[i].Set(Math.Floor(Math.Max(0.0, (ncolors-1)*(errs[i]-minerr)/(maxerr-minerr))+0.5));
			}
        }

		public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
		{
			// We're not plotting values, we're just displaying colors, so the scale will be constant:
			min = minLength;
			max = maxLength+1;
			return;
			
			// Normally, if we're plotting values, we'd get min and max like this:
			/*
			base.GetMinMaxValues(chartControl, ref min, ref max);
			int bars = base.ChartControl.BarsPainted;
			Exception caughtException;
            while (bars >= 0) {		
				int index = base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted + 1 + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired)) {
					try {
						double testval = Values[0].Get(index); // value of my indicator at bar 'index'
						if ((!double.IsNaN(testval)) && (index>0)) {
							if (testval>max) max=testval;
							if (testval<min) min=testval; 
						}
					}
					catch (Exception exception) { caughtException=exception; }
				}
				bars--;
			}
			*/
		}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			int barPaintWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			int bars = ChartControl.BarsPainted, index;
			double x1, y1, y2;
			double vscale = bounds.Height / ChartControl.MaxMinusMin(max, min);
			Exception caughtException;
			while (bars >= 0) {
				index = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + bars;
                if (ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired)) {
					try { // This 'try' block is where we plot something:
						x1 = // horizontal position of bar to plot
							(((ChartControl.CanvasRight - ChartControl.BarMarginRight) - (barPaintWidth / 2))
							- ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace))
							+ (bars * (base.ChartControl.BarSpace));
						for (int i = 0; i < nn; ++i) {
							y1 = (bounds.Y + bounds.Height) - (int)((i+minLength+1.5 - min) * vscale + 0.5);
							y2 = (bounds.Y + bounds.Height) - (int)((i+minLength - min) * vscale + 0.5);
							graphics.FillRectangle(brushcolor[(int)TLerrcolor[i].Get(index)], new Rectangle((int)x1-barPaintWidth/2, (int)y2, barPaintWidth, (int)Math.Abs(y1-y2)));
						}
					}
					catch (Exception exception) { caughtException = exception; }
				}
				bars--;
			}
			// we can draw other stuff here if we want (horizontal lines, shaded regions).
			// bounds.X, bounds.Y, bounds.Width, bounds.Height are dimensions of plotting area.
		}

        #region Properties
		[Description("Min length to return")]
        [Category("Parameters")]
        public int MinLength
        {
            get { return minLength; }
            set { minLength = Math.Max(3, value); }
        }

        [Description("Max length to return")]
        [Category("Parameters")]
        public int MaxLength
        {
            get { return maxLength; }
            set { maxLength = Math.Max(3, value); }
        }

		[Description("true=show grayscale, false=show colors")]
        [Category("Parameters")]
        public bool Grayscale
        {
            get { return grayscale; }
            set { grayscale = value; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TLfit[] cacheTLfit = null;

        private static TLfit checkTLfit = new TLfit();

        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        public TLfit TLfit(bool grayscale, int maxLength, int minLength)
        {
            return TLfit(Input, grayscale, maxLength, minLength);
        }

        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        public TLfit TLfit(Data.IDataSeries input, bool grayscale, int maxLength, int minLength)
        {
            checkTLfit.Grayscale = grayscale;
            grayscale = checkTLfit.Grayscale;
            checkTLfit.MaxLength = maxLength;
            maxLength = checkTLfit.MaxLength;
            checkTLfit.MinLength = minLength;
            minLength = checkTLfit.MinLength;

            if (cacheTLfit != null)
                for (int idx = 0; idx < cacheTLfit.Length; idx++)
                    if (cacheTLfit[idx].Grayscale == grayscale && cacheTLfit[idx].MaxLength == maxLength && cacheTLfit[idx].MinLength == minLength && cacheTLfit[idx].EqualsInput(input))
                        return cacheTLfit[idx];

            TLfit indicator = new TLfit();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Grayscale = grayscale;
            indicator.MaxLength = maxLength;
            indicator.MinLength = minLength;
            indicator.SetUp();

            TLfit[] tmp = new TLfit[cacheTLfit == null ? 1 : cacheTLfit.Length + 1];
            if (cacheTLfit != null)
                cacheTLfit.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTLfit = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TLfit TLfit(bool grayscale, int maxLength, int minLength)
        {
            return _indicator.TLfit(Input, grayscale, maxLength, minLength);
        }

        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        public Indicator.TLfit TLfit(Data.IDataSeries input, bool grayscale, int maxLength, int minLength)
        {
            return _indicator.TLfit(input, grayscale, maxLength, minLength);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TLfit TLfit(bool grayscale, int maxLength, int minLength)
        {
            return _indicator.TLfit(Input, grayscale, maxLength, minLength);
        }

        /// <summary>
        /// Displays color map of the quality of fit of all trendlines within a specified range of lengths.
        /// </summary>
        /// <returns></returns>
        public Indicator.TLfit TLfit(Data.IDataSeries input, bool grayscale, int maxLength, int minLength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TLfit(input, grayscale, maxLength, minLength);
        }

    }
}
#endregion
